@@ -1,5 +1,5 @@
-#ifndef _PLATFORM_H_
-#define _PLATFORM_H_
+#ifndef PLATFORM_H_
+#define PLATFORM_H_
 
 /* Ensure that we have a config file. */
 #if defined(CONFIG_H_FILE)
@@ -10,4 +10,4 @@
 #error Need either CONFIG_H_FILE or HAVE_CONFIG_H defined.
 #endif
 
-#endif /* !_PLATFORM_H_ */
+#endif /* !PLATFORM_H_ */
