@@ -4,12 +4,34 @@
 
 #include "cpusupport.h"
 #include "insecure_memzero.h"
+#include "sha256_arm.h"
 #include "sha256_shani.h"
+#include "sha256_sse2.h"
 #include "sysendian.h"
 #include "warnp.h"
 
 #include "sha256.h"
 
+#if defined(CPUSUPPORT_X86_SHANI) && defined(CPUSUPPORT_X86_SSSE3) ||	\
+    defined(CPUSUPPORT_X86_SSE2) ||					\
+    defined(CPUSUPPORT_ARM_SHA256)
+#define HWACCEL
+
+static enum {
+	HW_SOFTWARE = 0,
+#if defined(CPUSUPPORT_X86_SHANI) && defined(CPUSUPPORT_X86_SSSE3)
+	HW_X86_SHANI,
+#endif
+#if defined(CPUSUPPORT_X86_SSE2)
+	HW_X86_SSE2,
+#endif
+#if defined(CPUSUPPORT_ARM_SHA256)
+	HW_ARM_SHA256,
+#endif
+	HW_UNSET
+} hwaccel = HW_UNSET;
+#endif
+
 #ifdef POSIXFAIL_ABSTRACT_DECLARATOR
 static void SHA256_Transform(uint32_t state[static restrict 8],
     const uint8_t block[static restrict 64], uint32_t W[static restrict 64],
@@ -80,69 +102,99 @@ static const uint32_t initial_state[8] = {
 	0x510E527F, 0x9B05688C, 0x1F83D9AB, 0x5BE0CD19
 };
 
+#ifdef HWACCEL
 #if defined(CPUSUPPORT_X86_SHANI) && defined(CPUSUPPORT_X86_SSSE3)
+/* Shim so that we can test SHA256_Transform_shani() in the standard manner. */
+static void
+SHA256_Transform_shani_with_W_S(uint32_t state[static restrict 8],
+    const uint8_t block[static restrict 64], uint32_t W[static restrict 64],
+    uint32_t S[static restrict 8])
+{
+
+	(void)W; /* UNUSED */
+	(void)S; /* UNUSED */
+
+	SHA256_Transform_shani(state, block);
+}
+#endif
+#if defined(CPUSUPPORT_ARM_SHA256)
+/* Shim so that we can test SHA256_Transform_arm() in the standard manner. */
+static void
+SHA256_Transform_arm_with_W_S(uint32_t state[static restrict 8],
+    const uint8_t block[static restrict 64], uint32_t W[static restrict 64],
+    uint32_t S[static restrict 8])
+{
+
+	(void)W; /* UNUSED */
+	(void)S; /* UNUSED */
+
+	SHA256_Transform_arm(state, block);
+}
+#endif
+
 /*
- * Test whether software and SHANI transform code produce the same results.
- * Must be called with usesha() returning 0 (software).
+ * Test whether software and hardware extensions transform code produce the
+ * same results.  Must be called with (hwaccel == HW_SOFTWARE).
  */
 static int
-shanitest(const uint32_t state[static restrict 8],
+hwtest(const uint32_t state[static restrict 8],
     const uint8_t block[static restrict 64],
-    uint32_t W[static restrict 64], uint32_t S[static restrict 8])
+    uint32_t W[static restrict 64], uint32_t S[static restrict 8],
+    void(* func)(uint32_t state[static restrict 8],
+    const uint8_t block[static restrict 64], uint32_t W[static restrict 64],
+    uint32_t S[static restrict 8]))
 {
 	uint32_t state_sw[8];
-	uint32_t state_shani[8];
+	uint32_t state_hw[8];
 
 	/* Software transform. */
 	memcpy(state_sw, state, sizeof(state_sw));
 	SHA256_Transform(state_sw, block, W, S);
 
-	/* SHANI transform. */
-	memcpy(state_shani, state, sizeof(state_shani));
-	SHA256_Transform_shani(state_shani, block);
+	/* Hardware transform. */
+	memcpy(state_hw, state, sizeof(state_hw));
+	func(state_hw, block, W, S);
 
 	/* Do the results match? */
-	return (memcmp(state_sw, state_shani, sizeof(state_sw)));
+	return (memcmp(state_sw, state_hw, sizeof(state_sw)));
 }
 
-/* Should we use SHANI? */
-static int
-useshani(void)
+/* Which type of hardware acceleration should we use, if any? */
+static void
+hwaccel_init(void)
 {
-	static int shanigood = -1;
 	uint32_t W[64];
 	uint32_t S[8];
 	uint8_t block[64];
 	uint8_t i;
 
-	/* If we haven't decided which code to use yet, decide now. */
-	while (shanigood == -1) {
-		/* Default to software. */
-		shanigood = 0;
+	/* If we've already set hwaccel, we're finished. */
+	if (hwaccel != HW_UNSET)
+		return;
 
-		/* If the CPU doesn't claim to support AESNI, stop here. */
-		if (!cpusupport_x86_shani())
-			break;
+	/* Default to software. */
+	hwaccel = HW_SOFTWARE;
 
-		/* If the CPU doesn't claim to support SSSE3, stop here. */
-		if (!cpusupport_x86_ssse3())
-			break;
+	/* Test case: Hash 0x00 0x01 0x02 ... 0x3f. */
+	for (i = 0; i < 64; i++)
+		block[i] = i;
 
-		/* Test case: Hash 0x00 0x01 0x02 ... 0x3f. */
-		for (i = 0; i < 64; i++)
-			block[i] = i;
-		if (shanitest(initial_state, block, W, S)) {
-			warn0("Disabling SHANI due to failed self-test");
-			break;
-		}
-
-		/* SHANI works; use it. */
-		shanigood = 1;
-	}
-
-	return (shanigood);
+#if defined(CPUSUPPORT_X86_SHANI) && defined(CPUSUPPORT_X86_SSSE3)
+	CPUSUPPORT_VALIDATE(hwaccel, HW_X86_SHANI,
+	    cpusupport_x86_shani() && cpusupport_x86_ssse3(),
+	    hwtest(initial_state, block, W, S,
+		SHA256_Transform_shani_with_W_S));
+#endif
+#if defined(CPUSUPPORT_X86_SSE2)
+	CPUSUPPORT_VALIDATE(hwaccel, HW_X86_SSE2, cpusupport_x86_sse2(),
+	    hwtest(initial_state, block, W, S, SHA256_Transform_sse2));
+#endif
+#if defined(CPUSUPPORT_ARM_SHA256)
+	CPUSUPPORT_VALIDATE(hwaccel, HW_ARM_SHA256, cpusupport_arm_sha256(),
+	    hwtest(initial_state, block, W, S, SHA256_Transform_arm_with_W_S));
+#endif
 }
-#endif /* CPUSUPPORT_X86_SHANI && CPUSUPPORT_X86_SSSE3 */
+#endif /* HWACCEL */
 
 /* Elementary functions used by SHA256 */
 #define Ch(x, y, z)	((x & (y ^ z)) ^ z)
@@ -174,7 +226,8 @@ useshani(void)
 
 /*
  * SHA256 block compression function.  The 256-bit state is transformed via
- * the 512-bit input block to produce a new state.
+ * the 512-bit input block to produce a new state.  The arrays W and S may be
+ * filled with sensitive data, and should be sanitized by the callee.
  */
 static void
 SHA256_Transform(uint32_t state[static restrict 8],
@@ -183,13 +236,39 @@ SHA256_Transform(uint32_t state[static restrict 8],
 {
 	int i;
 
+#ifdef HWACCEL
+
+#if defined(__GNUC__) && defined(__aarch64__)
+	/*
+	 * We require that SHA256_Init() is called before SHA256_Transform(),
+	 * but the compiler has no way of knowing that.  This assert adds a
+	 * significant speed boost for gcc on 64-bit ARM, and a minor penalty
+	 * on other systems & compilers.
+	 */
+	assert(hwaccel != HW_UNSET);
+#endif
+
+	switch (hwaccel) {
 #if defined(CPUSUPPORT_X86_SHANI) && defined(CPUSUPPORT_X86_SSSE3)
-	/* Use SHANI if we can. */
-	if (useshani()) {
+	case HW_X86_SHANI:
 		SHA256_Transform_shani(state, block);
 		return;
-	}
 #endif
+#if defined(CPUSUPPORT_X86_SSE2)
+	case HW_X86_SSE2:
+		SHA256_Transform_sse2(state, block, W, S);
+		return;
+#endif
+#if defined(CPUSUPPORT_ARM_SHA256)
+	case HW_ARM_SHA256:
+		SHA256_Transform_arm(state, block);
+		return;
+#endif
+	case HW_SOFTWARE:
+	case HW_UNSET:
+		break;
+	}
+#endif /* HWACCEL */
 
 	/* 1. Prepare the first part of the message schedule W. */
 	be32dec_vect(W, block, 64);
@@ -290,6 +369,11 @@ SHA256_Init(SHA256_CTX * ctx)
 
 	/* Initialize state. */
 	memcpy(ctx->state, initial_state, sizeof(initial_state));
+
+#ifdef HWACCEL
+	/* Ensure that we've chosen the type of hardware acceleration. */
+	hwaccel_init();
+#endif
 }
 
 /**
@@ -297,7 +381,7 @@ SHA256_Init(SHA256_CTX * ctx)
  * Input ${len} bytes from ${in} into the SHA256 context ${ctx}.
  */
 static void
-_SHA256_Update(SHA256_CTX * ctx, const void * in, size_t len,
+SHA256_Update_internal(SHA256_CTX * ctx, const void * in, size_t len,
     uint32_t tmp32[static restrict 72])
 {
 	uint32_t r;
@@ -343,19 +427,19 @@ SHA256_Update(SHA256_CTX * ctx, const void * in, size_t len)
 	uint32_t tmp32[72];
 
 	/* Call the real function. */
-	_SHA256_Update(ctx, in, len, tmp32);
+	SHA256_Update_internal(ctx, in, len, tmp32);
 
 	/* Clean the stack. */
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 }
 
 /**
  * SHA256_Final(digest, ctx):
  * Output the SHA256 hash of the data input to the context ${ctx} into the
- * buffer ${digest}.
+ * buffer ${digest}, and clear the context state.
  */
 static void
-_SHA256_Final(uint8_t digest[32], SHA256_CTX * ctx,
+SHA256_Final_internal(uint8_t digest[32], SHA256_CTX * ctx,
     uint32_t tmp32[static restrict 72])
 {
 
@@ -373,13 +457,13 @@ SHA256_Final(uint8_t digest[32], SHA256_CTX * ctx)
 	uint32_t tmp32[72];
 
 	/* Call the real function. */
-	_SHA256_Final(digest, ctx, tmp32);
+	SHA256_Final_internal(digest, ctx, tmp32);
 
 	/* Clear the context state. */
 	insecure_memzero(ctx, sizeof(SHA256_CTX));
 
 	/* Clean the stack. */
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 }
 
 /**
@@ -393,12 +477,12 @@ SHA256_Buf(const void * in, size_t len, uint8_t digest[32])
 	uint32_t tmp32[72];
 
 	SHA256_Init(&ctx);
-	_SHA256_Update(&ctx, in, len, tmp32);
-	_SHA256_Final(digest, &ctx, tmp32);
+	SHA256_Update_internal(&ctx, in, len, tmp32);
+	SHA256_Final_internal(digest, &ctx, tmp32);
 
 	/* Clean the stack. */
 	insecure_memzero(&ctx, sizeof(SHA256_CTX));
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 }
 
 /**
@@ -407,18 +491,18 @@ SHA256_Buf(const void * in, size_t len, uint8_t digest[32])
  * ${K}.
  */
 static void
-_HMAC_SHA256_Init(HMAC_SHA256_CTX * ctx, const void * _K, size_t Klen,
+HMAC_SHA256_Init_internal(HMAC_SHA256_CTX * ctx, const void * _k, size_t Klen,
     uint32_t tmp32[static restrict 72], uint8_t pad[static restrict 64],
     uint8_t khash[static restrict 32])
 {
-	const uint8_t * K = _K;
+	const uint8_t * K = _k;
 	size_t i;
 
 	/* If Klen > 64, the key is really SHA256(K). */
 	if (Klen > 64) {
 		SHA256_Init(&ctx->ictx);
-		_SHA256_Update(&ctx->ictx, K, Klen, tmp32);
-		_SHA256_Final(khash, &ctx->ictx, tmp32);
+		SHA256_Update_internal(&ctx->ictx, K, Klen, tmp32);
+		SHA256_Final_internal(khash, &ctx->ictx, tmp32);
 		K = khash;
 		Klen = 32;
 	}
@@ -428,29 +512,29 @@ _HMAC_SHA256_Init(HMAC_SHA256_CTX * ctx, const void * _K, size_t Klen,
 	memset(pad, 0x36, 64);
 	for (i = 0; i < Klen; i++)
 		pad[i] ^= K[i];
-	_SHA256_Update(&ctx->ictx, pad, 64, tmp32);
+	SHA256_Update_internal(&ctx->ictx, pad, 64, tmp32);
 
 	/* Outer SHA256 operation is SHA256(K xor [block of 0x5c] || hash). */
 	SHA256_Init(&ctx->octx);
 	memset(pad, 0x5c, 64);
 	for (i = 0; i < Klen; i++)
 		pad[i] ^= K[i];
-	_SHA256_Update(&ctx->octx, pad, 64, tmp32);
+	SHA256_Update_internal(&ctx->octx, pad, 64, tmp32);
 }
 
 /* Wrapper function for intermediate-values sanitization. */
 void
-HMAC_SHA256_Init(HMAC_SHA256_CTX * ctx, const void * _K, size_t Klen)
+HMAC_SHA256_Init(HMAC_SHA256_CTX * ctx, const void * K, size_t Klen)
 {
 	uint32_t tmp32[72];
 	uint8_t pad[64];
 	uint8_t khash[32];
 
 	/* Call the real function. */
-	_HMAC_SHA256_Init(ctx, _K, Klen, tmp32, pad, khash);
+	HMAC_SHA256_Init_internal(ctx, K, Klen, tmp32, pad, khash);
 
 	/* Clean the stack. */
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 	insecure_memzero(khash, 32);
 	insecure_memzero(pad, 64);
 }
@@ -460,12 +544,12 @@ HMAC_SHA256_Init(HMAC_SHA256_CTX * ctx, const void * _K, size_t Klen)
  * Input ${len} bytes from ${in} into the HMAC-SHA256 context ${ctx}.
  */
 static void
-_HMAC_SHA256_Update(HMAC_SHA256_CTX * ctx, const void * in, size_t len,
+HMAC_SHA256_Update_internal(HMAC_SHA256_CTX * ctx, const void * in, size_t len,
     uint32_t tmp32[static restrict 72])
 {
 
 	/* Feed data to the inner SHA256 operation. */
-	_SHA256_Update(&ctx->ictx, in, len, tmp32);
+	SHA256_Update_internal(&ctx->ictx, in, len, tmp32);
 }
 
 /* Wrapper function for intermediate-values sanitization. */
@@ -475,30 +559,30 @@ HMAC_SHA256_Update(HMAC_SHA256_CTX * ctx, const void * in, size_t len)
 	uint32_t tmp32[72];
 
 	/* Call the real function. */
-	_HMAC_SHA256_Update(ctx, in, len, tmp32);
+	HMAC_SHA256_Update_internal(ctx, in, len, tmp32);
 
 	/* Clean the stack. */
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 }
 
 /**
  * HMAC_SHA256_Final(digest, ctx):
  * Output the HMAC-SHA256 of the data input to the context ${ctx} into the
- * buffer ${digest}.
+ * buffer ${digest}, and clear the context state.
  */
 static void
-_HMAC_SHA256_Final(uint8_t digest[32], HMAC_SHA256_CTX * ctx,
+HMAC_SHA256_Final_internal(uint8_t digest[32], HMAC_SHA256_CTX * ctx,
     uint32_t tmp32[static restrict 72], uint8_t ihash[static restrict 32])
 {
 
 	/* Finish the inner SHA256 operation. */
-	_SHA256_Final(ihash, &ctx->ictx, tmp32);
+	SHA256_Final_internal(ihash, &ctx->ictx, tmp32);
 
 	/* Feed the inner hash to the outer SHA256 operation. */
-	_SHA256_Update(&ctx->octx, ihash, 32, tmp32);
+	SHA256_Update_internal(&ctx->octx, ihash, 32, tmp32);
 
 	/* Finish the outer SHA256 operation. */
-	_SHA256_Final(digest, &ctx->octx, tmp32);
+	SHA256_Final_internal(digest, &ctx->octx, tmp32);
 }
 
 /* Wrapper function for intermediate-values sanitization. */
@@ -509,13 +593,13 @@ HMAC_SHA256_Final(uint8_t digest[32], HMAC_SHA256_CTX * ctx)
 	uint8_t ihash[32];
 
 	/* Call the real function. */
-	_HMAC_SHA256_Final(digest, ctx, tmp32, ihash);
+	HMAC_SHA256_Final_internal(digest, ctx, tmp32, ihash);
 
 	/* Clear the context state. */
 	insecure_memzero(ctx, sizeof(HMAC_SHA256_CTX));
 
 	/* Clean the stack. */
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 	insecure_memzero(ihash, 32);
 }
 
@@ -532,13 +616,13 @@ HMAC_SHA256_Buf(const void * K, size_t Klen, const void * in, size_t len,
 	uint32_t tmp32[72];
 	uint8_t tmp8[96];
 
-	_HMAC_SHA256_Init(&ctx, K, Klen, tmp32, &tmp8[0], &tmp8[64]);
-	_HMAC_SHA256_Update(&ctx, in, len, tmp32);
-	_HMAC_SHA256_Final(digest, &ctx, tmp32, &tmp8[0]);
+	HMAC_SHA256_Init_internal(&ctx, K, Klen, tmp32, &tmp8[0], &tmp8[64]);
+	HMAC_SHA256_Update_internal(&ctx, in, len, tmp32);
+	HMAC_SHA256_Final_internal(digest, &ctx, tmp32, &tmp8[0]);
 
 	/* Clean the stack. */
 	insecure_memzero(&ctx, sizeof(HMAC_SHA256_CTX));
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 	insecure_memzero(tmp8, 96);
 }
 
@@ -562,16 +646,18 @@ PBKDF2_SHA256(const uint8_t * passwd, size_t passwdlen, const uint8_t * salt,
 	int k;
 	size_t clen;
 
+#if SIZE_MAX >= (32 * UINT32_MAX)
 	/* Sanity-check. */
 	assert(dkLen <= 32 * (size_t)(UINT32_MAX));
+#endif
 
 	/* Compute HMAC state after processing P. */
-	_HMAC_SHA256_Init(&Phctx, passwd, passwdlen,
+	HMAC_SHA256_Init_internal(&Phctx, passwd, passwdlen,
 	    tmp32, &tmp8[0], &tmp8[64]);
 
 	/* Compute HMAC state after processing P and S. */
 	memcpy(&PShctx, &Phctx, sizeof(HMAC_SHA256_CTX));
-	_HMAC_SHA256_Update(&PShctx, salt, saltlen, tmp32);
+	HMAC_SHA256_Update_internal(&PShctx, salt, saltlen, tmp32);
 
 	/* Iterate through the blocks. */
 	for (i = 0; i * 32 < dkLen; i++) {
@@ -580,8 +666,8 @@ PBKDF2_SHA256(const uint8_t * passwd, size_t passwdlen, const uint8_t * salt,
 
 		/* Compute U_1 = PRF(P, S || INT(i)). */
 		memcpy(&hctx, &PShctx, sizeof(HMAC_SHA256_CTX));
-		_HMAC_SHA256_Update(&hctx, ivec, 4, tmp32);
-		_HMAC_SHA256_Final(U, &hctx, tmp32, tmp8);
+		HMAC_SHA256_Update_internal(&hctx, ivec, 4, tmp32);
+		HMAC_SHA256_Final_internal(U, &hctx, tmp32, tmp8);
 
 		/* T_i = U_1 ... */
 		memcpy(T, U, 32);
@@ -589,8 +675,8 @@ PBKDF2_SHA256(const uint8_t * passwd, size_t passwdlen, const uint8_t * salt,
 		for (j = 2; j <= c; j++) {
 			/* Compute U_j. */
 			memcpy(&hctx, &Phctx, sizeof(HMAC_SHA256_CTX));
-			_HMAC_SHA256_Update(&hctx, U, 32, tmp32);
-			_HMAC_SHA256_Final(U, &hctx, tmp32, tmp8);
+			HMAC_SHA256_Update_internal(&hctx, U, 32, tmp32);
+			HMAC_SHA256_Final_internal(U, &hctx, tmp32, tmp8);
 
 			/* ... xor U_j ... */
 			for (k = 0; k < 32; k++)
@@ -608,7 +694,7 @@ PBKDF2_SHA256(const uint8_t * passwd, size_t passwdlen, const uint8_t * salt,
 	insecure_memzero(&Phctx, sizeof(HMAC_SHA256_CTX));
 	insecure_memzero(&PShctx, sizeof(HMAC_SHA256_CTX));
 	insecure_memzero(&hctx, sizeof(HMAC_SHA256_CTX));
-	insecure_memzero(tmp32, 288);
+	insecure_memzero(tmp32, sizeof(uint32_t) * 72);
 	insecure_memzero(tmp8, 96);
 	insecure_memzero(U, 32);
 	insecure_memzero(T, 32);
