@@ -1,10 +1,11 @@
-#ifndef _MONOCLOCK_H_
-#define _MONOCLOCK_H_
+#ifndef MONOCLOCK_H_
+#define MONOCLOCK_H_
 
 #include <sys/time.h>
 
-#define timeval_diff(x, y) ((y.tv_sec - x.tv_sec) +			\
-    (y.tv_usec - x.tv_usec) * 0.000001)
+/* Macro to simplify benchmarks. */
+#define timeval_diff(x, y) ((double)((y).tv_sec - (x).tv_sec) +		\
+    (double)((y).tv_usec - (x).tv_usec) * 0.000001)
 
 /**
  * monoclock_get(tv):
@@ -32,4 +33,4 @@ int monoclock_get_cputime(struct timeval *);
  */
 int monoclock_getres(double *);
 
-#endif /* !_MONOCLOCK_H_ */
+#endif /* !MONOCLOCK_H_ */
