@@ -36,20 +36,11 @@
 #include "humansize.h"
 #include "insecure_memzero.h"
 #include "parsenum.h"
-#include "readpass.h"
+#include "passphrase_entry.h"
 #include "scryptenc.h"
+#include "scryptenc_print_error.h"
 #include "warnp.h"
 
-/* How should we get the passphrase? */
-enum passphrase_entry {
-	PASSPHRASE_UNSET,
-	PASSPHRASE_TTY_STDIN,
-	PASSPHRASE_STDIN_ONCE,
-	PASSPHRASE_TTY_ONCE,
-	PASSPHRASE_ENV,
-	PASSPHRASE_FILE,
-};
-
 static void
 usage(void)
 {
@@ -63,6 +54,156 @@ usage(void)
 	exit(1);
 }
 
+/**
+ * scrypt_mode_info(infilename):
+ * Print scrypt parameters used for the specified ${infilename}, or read from
+ * stdin if that argument is NULL.
+ */
+static int
+scrypt_mode_info(const char * infilename)
+{
+	FILE * infile;
+	int rc;
+
+	/* If the input isn't stdin, open the file. */
+	if (infilename != NULL) {
+		if ((infile = fopen(infilename, "rb")) == NULL) {
+			warnp("Cannot open input file: %s", infilename);
+			goto err0;
+		}
+	} else {
+		infile = stdin;
+	}
+
+	/* Print the encryption parameters used for the file. */
+	if ((rc = scryptdec_file_printparams(infile)) != SCRYPT_OK) {
+		scryptenc_print_error(rc, infilename, NULL);
+		goto err1;
+	}
+
+	/* Clean up. */
+	if ((infile != stdin) && fclose(infile))
+		warnp("fclose");
+
+	/* Success! */
+	return (0);
+
+err1:
+	if ((infile != stdin) && fclose(infile))
+		warnp("fclose");
+err0:
+	/* Failure! */
+	return (-1);
+}
+
+/**
+ * scrypt_mode_enc_dec(params, passphrase_entry, passphrase_arg, dec, verbose,
+ *     force_resources, infilename, outfilename):
+ * Either encrypt (if ${dec} is 0) or decrypt (if ${dec} is non-zero)
+ * ${infilename} (or standard input if this is NULL) to ${outfilename}.
+ * Use scrypt parameters ${params}, with passphrase entry method
+ * ${passphrase_entry} and argument ${passphrase_arg}.  If ${verbose} is
+ * non-zero, print verbose messages.  If ${force_resources} is non-zero,
+ * do not check whether encryption or decryption will exceed the estimated
+ * time or memory requirements.
+ */
+static int
+scrypt_mode_enc_dec(struct scryptenc_params params,
+    enum passphrase_entry passphrase_entry, const char * passphrase_arg,
+    int dec, int verbose, int force_resources,
+    const char * infilename, const char * outfilename)
+{
+	struct scryptdec_file_cookie * C = NULL;
+	FILE * infile;
+	FILE * outfile = stdout;
+	char * passwd;
+	int rc;
+
+	/* If the input isn't stdin, open the file. */
+	if (infilename != NULL) {
+		if ((infile = fopen(infilename, "rb")) == NULL) {
+			warnp("Cannot open input file: %s", infilename);
+			goto err0;
+		}
+	} else {
+		infile = stdin;
+	}
+
+	/* Get the password. */
+	if (passphrase_entry_readpass(&passwd, passphrase_entry,
+	    passphrase_arg, "Please enter passphrase",
+	    "Please confirm passphrase", dec)) {
+		warnp("passphrase_entry_readpass");
+		goto err1;
+	}
+
+	/*-
+	 * If we're decrypting, open the input file and process its header;
+	 * doing this here allows us to abort without creating an output
+	 * file if the input file does not have a valid scrypt header or if
+	 * we have the wrong passphrase.
+	 *
+	 * If successful, we get back a cookie containing the decryption
+	 * parameters (which we'll use after we open the output file).
+	 */
+	if (dec) {
+		if ((rc = scryptdec_file_prep(infile, (uint8_t *)passwd,
+		    strlen(passwd), &params, verbose, force_resources,
+		    &C)) != 0) {
+			goto cleanup;
+		}
+	}
+
+	/* If we have an output filename, open it. */
+	if (outfilename != NULL) {
+		if ((outfile = fopen(outfilename, "wb")) == NULL) {
+			warnp("Cannot open output file: %s", outfilename);
+			goto err2;
+		}
+	}
+
+	/* Encrypt or decrypt. */
+	if (dec)
+		rc = scryptdec_file_copy(C, outfile);
+	else
+		rc = scryptenc_file(infile, outfile, (uint8_t *)passwd,
+		    strlen(passwd), &params, verbose, force_resources);
+
+cleanup:
+	/* Free the decryption cookie, if any. */
+	scryptdec_file_cookie_free(C);
+
+	/* Zero and free the password. */
+	insecure_memzero(passwd, strlen(passwd));
+	free(passwd);
+
+	/* Close any files we opened. */
+	if ((infile != stdin) && fclose(infile))
+		warnp("fclose");
+	if ((outfile != stdout) && fclose(outfile))
+		warnp("fclose");
+
+	/* If we failed, print the right error message and exit. */
+	if (rc != SCRYPT_OK) {
+		scryptenc_print_error(rc, infilename, outfilename);
+		goto err0;
+	}
+
+	/* Success! */
+	return (0);
+
+err2:
+	scryptdec_file_cookie_free(C);
+	insecure_memzero(passwd, strlen(passwd));
+	free(passwd);
+err1:
+	if ((infile != stdin) && fclose(infile))
+		warnp("fclose");
+err0:
+	/* Failure! */
+	return (-1);
+}
+
 /* Parse a numeric optarg within a GETOPT context.  (Requires ch and optarg.) */
 #define GETOPT_PARSENUM_WITHIN_UNSIGNED(var, min, max) do {		\
 	if (PARSENUM((var), optarg, (min), (max))) {			\
@@ -74,61 +215,11 @@ usage(void)
 			warnp("Invalid option: %s %s", ch, optarg);	\
 		exit(1);						\
 	}								\
-} while(0)
-
-static int
-parse_passphrase_arg(const char * arg,
-    enum passphrase_entry * passphrase_entry_p, const char ** passphrase_arg_p)
-{
-	const char * p;
-
-	/* Find the separator in "method:arg", or fail if there isn't one. */
-	if ((p = strchr(arg, ':')) == NULL)
-		goto err1;
-
-	/* Extract the "arg" part. */
-	*passphrase_arg_p = &p[1];
-
-	/* Parse the "method". */
-	if (strncmp(arg, "dev:", 4) == 0) {
-		if (strcmp(*passphrase_arg_p, "tty-stdin") == 0) {
-			*passphrase_entry_p = PASSPHRASE_TTY_STDIN;
-			goto success;
-		}
-		else if (strcmp(*passphrase_arg_p, "stdin-once") == 0) {
-			*passphrase_entry_p = PASSPHRASE_STDIN_ONCE;
-			goto success;
-		}
-		else if (strcmp(*passphrase_arg_p, "tty-once") == 0) {
-			*passphrase_entry_p = PASSPHRASE_TTY_ONCE;
-			goto success;
-		}
-	}
-	if (strncmp(optarg, "env:", 4) == 0) {
-		*passphrase_entry_p = PASSPHRASE_ENV;
-		goto success;
-	}
-	if (strncmp(optarg, "file:", 5) == 0) {
-		*passphrase_entry_p = PASSPHRASE_FILE;
-		goto success;
-	}
-
-err1:
-	warn0("Invalid option: --passphrase %s", arg);
-
-	/* Failure! */
-	return (-1);
-
-success:
-	/* Success! */
-	return (0);
-}
+} while (0)
 
 int
-main(int argc, char *argv[])
+main(int argc, char * argv[])
 {
-	FILE * infile;
-	FILE * outfile = stdout;
 	int dec = 0;
 	int info = 0;
 	int force_resources = 0;
@@ -137,13 +228,9 @@ main(int argc, char *argv[])
 	const char * ch;
 	const char * infilename;
 	const char * outfilename;
-	char * passwd;
-	int rc;
 	int verbose = 0;
-	struct scryptdec_file_cookie * C = NULL;
 	enum passphrase_entry passphrase_entry = PASSPHRASE_UNSET;
 	const char * passphrase_arg;
-	const char * passwd_env;
 
 	WARNP_INIT;
 
@@ -189,7 +276,7 @@ main(int argc, char *argv[])
 			params.maxmem = (size_t)maxmem64;
 			break;
 		GETOPT_OPTARG("-m"):
-			if (PARSENUM(&params.maxmemfrac, optarg, 0, 1)) {
+			if (PARSENUM(&params.maxmemfrac, optarg, 0, 0.5)) {
 				warnp("Invalid option: -m %s", optarg);
 				exit(1);
 			}
@@ -205,7 +292,7 @@ main(int argc, char *argv[])
 			}
 
 			/* Parse "method:arg" optarg. */
-			if (parse_passphrase_arg(optarg, &passphrase_entry,
+			if (passphrase_entry_parse(optarg, &passphrase_entry,
 			    &passphrase_arg))
 				exit(1);
 			break;
@@ -228,6 +315,7 @@ main(int argc, char *argv[])
 				exit(1);
 			}
 			passphrase_entry = PASSPHRASE_STDIN_ONCE;
+			passphrase_arg = "";
 			break;
 		GETOPT_MISSING_ARG:
 			warn0("Missing argument to %s", ch);
@@ -258,6 +346,12 @@ main(int argc, char *argv[])
 		goto err0;
 	}
 
+	/* We can't have a maxmemfrac of 0. */
+	if (params.maxmemfrac == 0.0) {
+		warn0("-m must be greater than 0");
+		goto err0;
+	}
+
 	/* Set the input filename. */
 	if (strcmp(argv[0], "-"))
 		infilename = argv[0];
@@ -271,189 +365,35 @@ main(int argc, char *argv[])
 		outfilename = NULL;
 
 	/* Set the default passphrase entry method. */
-	if (passphrase_entry == PASSPHRASE_UNSET)
+	if (passphrase_entry == PASSPHRASE_UNSET) {
 		passphrase_entry = PASSPHRASE_TTY_STDIN;
-
-	/* If the input isn't stdin, open the file. */
-	if (infilename != NULL) {
-		if ((infile = fopen(infilename, "rb")) == NULL) {
-			warnp("Cannot open input file: %s", infilename);
-			goto err0;
-		}
-	} else {
-		infile = stdin;
-
-		/* Error if given incompatible options. */
-		if (passphrase_entry == PASSPHRASE_STDIN_ONCE) {
-			warn0("Cannot read both passphrase and input file"
-			    " from standard input");
-			goto err0;
-		}
+		passphrase_arg = "";
 	}
 
-	/* User selected 'info' mode. */
-	if (info) {
-		/* Print the encryption parameters used for the file. */
-		rc = scryptdec_file_printparams(infile);
-
-		/* Clean up. */
-		if (infile != stdin)
-			fclose(infile);
-
-		/* Finished! */
-		goto done;
-	}
-
-	/* Get the password. */
-	switch (passphrase_entry) {
-	case PASSPHRASE_TTY_STDIN:
-		/* Read passphrase, prompting only once if decrypting. */
-		if (readpass(&passwd, "Please enter passphrase",
-		    (dec) ? NULL : "Please confirm passphrase", 1))
-			goto err1;
-		break;
-	case PASSPHRASE_STDIN_ONCE:
-		/* Read passphrase, prompting only once, from stdin only. */
-		if (readpass(&passwd, "Please enter passphrase", NULL, 0))
-			goto err1;
-		break;
-	case PASSPHRASE_TTY_ONCE:
-		/* Read passphrase, prompting only once, from tty only. */
-		if (readpass(&passwd, "Please enter passphrase", NULL, 2))
-			goto err1;
-		break;
-	case PASSPHRASE_ENV:
-		/* We're not allowed to modify the output of getenv(). */
-		if ((passwd_env = getenv(passphrase_arg)) == NULL) {
-			warn0("Failed to read from ${%s}", passphrase_arg);
-			goto err1;
-		}
-
-		/* This allows us to use the same insecure_zero() logic. */
-		if ((passwd = strdup(passwd_env)) == NULL) {
-			warnp("Out of memory");
-			goto err1;
-		}
-		break;
-	case PASSPHRASE_FILE:
-		if (readpass_file(&passwd, passphrase_arg))
-			goto err1;
-		break;
-	case PASSPHRASE_UNSET:
-		warn0("Programming error: passphrase_entry is not set");
-		goto err1;
-	}
-
-	/*-
-	 * If we're decrypting, open the input file and process its header;
-	 * doing this here allows us to abort without creating an output
-	 * file if the input file does not have a valid scrypt header or if
-	 * we have the wrong passphrase.
-	 *
-	 * If successful, we get back a cookie containing the decryption
-	 * parameters (which we'll use after we open the output file).
-	 */
-	if (dec) {
-		if ((rc = scryptdec_file_prep(infile, (uint8_t *)passwd,
-		    strlen(passwd), &params, verbose, force_resources,
-		    &C)) != 0) {
-			goto cleanup;
-		}
-	}
-
-	/* If we have an output file, open it. */
-	if (outfilename != NULL) {
-		if ((outfile = fopen(outfilename, "wb")) == NULL) {
-			warnp("Cannot open output file: %s", outfilename);
-			goto err2;
-		}
-	}
-
-	/* Encrypt or decrypt. */
-	if (dec)
-		rc = scryptdec_file_copy(C, outfile);
-	else
-		rc = scryptenc_file(infile, outfile, (uint8_t *)passwd,
-		    strlen(passwd), &params, verbose, force_resources);
-
-cleanup:
-	/* Free the decryption cookie, if any. */
-	scryptdec_file_cookie_free(C);
-
-	/* Zero and free the password. */
-	insecure_memzero(passwd, strlen(passwd));
-	free(passwd);
-
-	/* Close any files we opened. */
-	if (infile != stdin)
-		fclose(infile);
-	if (outfile != stdout)
-		fclose(outfile);
-
-done:
-	/* If we failed, print the right error message and exit. */
-	if (rc != SCRYPT_OK) {
-		switch (rc) {
-		case SCRYPT_ELIMIT:
-			warnp("Error determining amount of available memory");
-			break;
-		case SCRYPT_ECLOCK:
-			warnp("Error reading clocks");
-			break;
-		case SCRYPT_EKEY:
-			warnp("Error computing derived key");
-			break;
-		case SCRYPT_ESALT:
-			warnp("Error reading salt");
-			break;
-		case SCRYPT_EOPENSSL:
-			warnp("OpenSSL error");
-			break;
-		case SCRYPT_ENOMEM:
-			warnp("Error allocating memory");
-			break;
-		case SCRYPT_EINVAL:
-			warn0("Input is not valid scrypt-encrypted block");
-			break;
-		case SCRYPT_EVERSION:
-			warn0("Unrecognized scrypt format version");
-			break;
-		case SCRYPT_ETOOBIG:
-			warn0("Decrypting file would require too much memory");
-			break;
-		case SCRYPT_ETOOSLOW:
-			warn0("Decrypting file would take too much CPU time");
-			break;
-		case SCRYPT_EPASS:
-			warn0("Passphrase is incorrect");
-			break;
-		case SCRYPT_EWRFILE:
-			warnp("Error writing file: %s",
-			    (outfilename != NULL) ? outfilename
-			    : "standard output");
-			break;
-		case SCRYPT_ERDFILE:
-			warnp("Error reading file: %s",
-			    (infilename != NULL) ? infilename
-			    : "standard input");
-			break;
-		case SCRYPT_EPARAM:
-			warn0("Error in explicit parameters");
-			break;
-		}
+	/* Sanity check passphrase entry method and input filename. */
+	if ((passphrase_entry == PASSPHRASE_STDIN_ONCE) &&
+	    (infilename == NULL)) {
+		warn0("Cannot read both passphrase and input file"
+		    " from standard input");
 		goto err0;
 	}
 
+	/* What type of operation are we doing? */
+	if (info) {
+		/* User selected 'info' mode. */
+		if (scrypt_mode_info(infilename))
+			goto err0;
+	} else {
+		/* User selected encryption or decryption. */
+		if (scrypt_mode_enc_dec(params, passphrase_entry,
+		    passphrase_arg, dec, verbose, force_resources,
+		    infilename, outfilename))
+			goto err0;
+	}
+
 	/* Success! */
-	return (0);
+	exit(0);
 
-err2:
-	scryptdec_file_cookie_free(C);
-	insecure_memzero(passwd, strlen(passwd));
-	free(passwd);
-err1:
-	if (infile != stdin)
-		fclose(infile);
 err0:
 	/* Failure! */
 	exit(1);
